/*
 * @(#)file      SnmpJdmkProxyLcd.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.11
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.v3proxy;

import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyLcd;
import com.sun.management.snmp.rfc2573.SnmpJdmkRfc2573ProxyForwarder;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.internal.SnmpAddressFactory;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpAgentProxy;
import java.util.Enumeration;

public class SnmpJdmkProxyLcd implements SnmpProxyLcd {
    SnmpProxiesReader proxies = null;
    SnmpProxyFactory proxyFactory = null;
    SnmpAddressFactory addressFactory = null;
    SnmpJdmkRfc2573ProxyForwarder appli = null;
    SnmpTargetLcd targetLcd = null;
    String proxiesFile = null;
    boolean init = false;
    SnmpEngine engine = null;
    SnmpV3AdaptorServer server = null;
    public SnmpJdmkProxyLcd(SnmpV3AdaptorServer server,
			    SnmpJdmkRfc2573ProxyForwarder appli,
			    SnmpEngine engine,
			    String proxiesFile,
			    SnmpTargetLcd targetLcd) {
	this.proxiesFile = proxiesFile;
	this.targetLcd = targetLcd;
	this.engine = engine;
	this.appli = appli;
	this.server = server;
    }

    /**
     * Enumeration of <CODE>SnmpAgentProxy</CODE>
     */
    public Enumeration getProxies() {
	return proxies.getProxies();
    }

    public void removeProxy(String name) throws IllegalArgumentException {
	proxies.removeProxy(name);
    }
    
    public void addProxy(String proxyName,
			 int type,
			 SnmpEngineId contextEngineId,
			 String contextName,
			 String paramsIn,
			 String singleOut,
			  String multipleOut,
			 int storageType) throws IllegalArgumentException {
	proxies.addProxy(proxyName,
			 type,
			 contextEngineId,
			 contextName,
			 paramsIn,
			 singleOut,
			 multipleOut,
			 storageType);
	proxies.flushInFile();
    }

    /**
     * When a request is received, the proxy that will handle the call is serach.
     */
    public SnmpAgentProxy lookUpProxy(int type,
				      int pduType,
				      SnmpEngineId ctxEngineId,
				      String ctxName,
				      int mpm,
				      int securityModel,
				      String securityName,
				      int securityLevel) {
	SnmpProxyData data =  proxies.findProxy(ctxEngineId,
						ctxName,
						type,
						pduType,
						mpm, 
						securityModel,
						securityName,
						securityLevel);
	return data.proxy;
    }
    

    public void readConfiguration() {
	if(init) return;

	if(proxyFactory == null) 
	    proxyFactory = new SnmpProxyFactory();
	
	//proxy need params + targets
	proxies = new SnmpProxiesReader(engine,
					this,
					proxyFactory,
					proxiesFile,
					targetLcd);
	init = true;
    }

    /**
     * To allow your own proxy creation.
     */
    public void setProxyFactory(SnmpProxyFactory fact) {
	proxyFactory = fact;
    }

    public SnmpProxyFactory getProxyFactory() {
	return proxyFactory;
    }
    
    /**
     * To allow your own proxy creation.
     */
    public void setAddressFactory(SnmpAddressFactory fact) {
	addressFactory = fact;
    }

    public SnmpAddressFactory getAddressFactory() {
	return addressFactory;
    }
    
    void addContextEngineIdProxy(SnmpEngineId contextEngineId, int type) {
	if(isDebugOn()) {
	    debug("addContextEngineIdProxy", 
		  "Registering for: " + contextEngineId);
	}
	try {
	    server.addRequestForwarder(appli, contextEngineId);
	}catch(IllegalArgumentException e) {
	    if(isDebugOn()) {
		debug("addContextEngineIdProxy", 
		      "Already registered: " + contextEngineId);
	    }
	}
    }
        private boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP);
    }

    private void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    private boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP);
    }

    private void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    String dbgTag = "SnmpJdmkProxyLcd";
}
