/*
 * @(#)file      SnmpProxyFactory.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */

package com.sun.management.snmp.rfc2573.internal.v3proxy;

import com.sun.management.snmp.agent.SnmpProxy;
import com.sun.management.snmp.agent.SnmpUsmProxy;
import com.sun.management.snmp.agent.SnmpV3Proxy;
import com.sun.management.snmp.usm.SnmpUsm;
import com.sun.management.snmp.SnmpEngine;
import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.rfc2573.internal.SnmpIpDomainsAddress;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;
import com.sun.management.snmp.SnmpDefinitions;

public class SnmpProxyFactory {
    
    public SnmpProxyFactory() {
    }
    /**
     * Overload this method in order to instanciate your own proxy. This method creates <CODE> SnmpProxy </CODE> for SNMP V1 and SNMP V2. <CODE> SnmpUsmProxy </CODE> for SNMP V3 and Usm.
     * @param mpm Message Processing Model.
     * @param securityModel Security model.
     * @param address Peer address.
     * @param port Peer port.
     * @return The peer. 
     */
    public SnmpProxy createProxy(SnmpEngine engine,
				 SnmpProxyData d) throws Exception {
	if(! ( d.singleOut.address.getDomain().equals(SnmpIpDomainsAddress.UDP_DOMAIN) ) )
	    return null;
	
	int mpm = d.singleOut.params.mpm;
	int securityModel = d.singleOut.params.securityModel;
	String name = d.singleOut.params.securityName;
	SnmpProxy proxy = null;
	if(isDebugOn()) 
	    debug("createProxy","Creating SNMP version : mpm [" + mpm + "] security model [" + securityModel +"]");
	switch(mpm) {
	case SnmpDefinitions.snmpVersionOne:
	case SnmpDefinitions.snmpVersionTwo:
	    switch(securityModel) {
	    case SnmpDefinitions.snmpVersionOne:
	    case SnmpDefinitions.snmpVersionTwo:
		if(isDebugOn()) 
		    debug("createProxy","Creating SnmpRfc2573Proxy("+d+")");
		proxy = new SnmpRfc2573Proxy(d,engine);
		break;
	    default:
	    }
	    break;
	case SnmpDefinitions.snmpVersionThree:
	    switch(securityModel) {
	    case SnmpUsm.ID:
		if(isDebugOn()) 
		    debug("createProxy","Creating SnmpRfc2573UsmProxy("+d+")");
		proxy = new SnmpRfc2573UsmProxy(d,engine);
		break;
	    default:
	    }
	    break;
	default:
	}
	return proxy;
    }
        // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpProxyFactory"; 
}
