/*
 * @(#)file      SnmpTargetsParamsHandler.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.13
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.manager.internal.target;

// Util import
import java.util.Enumeration;

// JMX RI
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;

// JDMK import
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.TraceManager;

import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.management.snmp.EnumRowStatus;

// CONTRIB MIB import
import com.sun.management.snmp.rfc2573.target.mib.SNMP_TARGET_MIBOidTable;
import com.sun.management.snmp.rfc2573.internal.SnmpIpDomainsAddress;
// CONTRIB MGR import
import com.sun.management.snmp.rfc2573.manager.internal.target.SnmpParamsData;
import com.sun.management.snmp.rfc2573.manager.internal.target.SnmpTargetData;

public class SnmpTargetsParamsHandler  {
        
    public static void setParams(SnmpSession session,
			  SnmpParamsData data) 

	throws SnmpStatusException {
		
	SnmpVarBindList list = new SnmpVarBindList();
 
	//Name Index
	SnmpString name = new SnmpString(data.paramsName);
	SnmpOid nameOid = name.toOid();
	long length = data.paramsName.length();

 	// set the Processing model
	SnmpOid oidProcessingModel = new SnmpOid("1.3.6.1.6.3.12.1.3.1.2");
	SnmpInt processingModel = new SnmpInt(data.mpm);
	oidProcessingModel.append(length);
	oidProcessingModel.append(nameOid);
	if(isDebugOn())
	    debug("setParams", oidProcessingModel.toString());
	list.addVarBind(new SnmpVarBind(oidProcessingModel, processingModel));

	// Set for Security Model 
	SnmpOid oidSecurityModel = new SnmpOid("1.3.6.1.6.3.12.1.3.1.3");
	SnmpInt securityModel = new SnmpInt(data.securityModel);
	oidSecurityModel.append(length);
	oidSecurityModel.append(nameOid);
	if(isDebugOn())
	    debug("setParams", oidSecurityModel.toString());
	list.addVarBind(new SnmpVarBind(oidSecurityModel, securityModel));

	// Set for Security Name
	SnmpOid oidSecurityName = new SnmpOid("1.3.6.1.6.3.12.1.3.1.4");
	SnmpString securityName = new SnmpString(data.securityName);
	oidSecurityName.append(length);
	oidSecurityName.append(nameOid);
	if(isDebugOn())
	    debug("setParams", oidSecurityName.toString());
	list.addVarBind(new SnmpVarBind(oidSecurityName, securityName));

	// Set for the security Level
	SnmpOid oidLevel = new SnmpOid("1.3.6.1.6.3.12.1.3.1.5");
	SnmpInt level = new SnmpInt(data.securityLevel);
	oidLevel.append(length);
	oidLevel.append(nameOid);
	if(isDebugOn())
	    debug("setParams", oidLevel.toString());
	list.addVarBind(new SnmpVarBind(oidLevel, level));

	//storage
	SnmpOid oidStorage = new SnmpOid("1.3.6.1.6.3.12.1.3.1.6");
	SnmpInt storage = new SnmpInt(data.storageType);
	oidStorage.append(length);
	oidStorage.append(nameOid);
	if(isDebugOn())
	    debug("setParams", oidStorage.toString());
	list.addVarBind(new SnmpVarBind(oidStorage, storage));
	
	//RowStatus
	SnmpOid oidRowStatus = new SnmpOid("1.3.6.1.6.3.12.1.3.1.7");
	oidRowStatus.append(length);
	oidRowStatus.append(nameOid);
	EnumRowStatus createAndGo = new EnumRowStatus("createAndGo");
	if(isDebugOn())
	    debug("setParams", oidRowStatus.toString());
	list.addVarBind(new SnmpVarBind(oidRowStatus, 
					createAndGo.toSnmpValue()));
	
	// Send the SET request
	SnmpRequest request = session.snmpSetRequest(null, list);

	boolean completed = request.waitForCompletion(session.getDefaultPeer().getTimeout());
	if (completed == false) {
	  if (isTraceOn())
	      trace("register", "Request timed out: check reachability of agent.\n");
	    throw new SnmpStatusException(SnmpDefinitions.snmpReqTimeout, 
					  0);
	}

	// Check the status
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    if (isTraceOn()) {
		trace("register","************* Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
		trace("register","Error index = " + request.getErrorIndex());
		throw new SnmpStatusException(errorStatus,
					      request.getErrorIndex());
	    }
	} else {
	    if (isDebugOn()) {
		SnmpVarBindList result = request.getResponseVarBindList();
		debug("register","Result: \n" + result.varBindListToString());
	    }
	}
    }

    public static void removeParams(SnmpSession session, 
			     String paramsIndex) 
	throws SnmpStatusException {
		
	SnmpVarBindList list = new SnmpVarBindList();

	//Name Index
	SnmpString name = new SnmpString(paramsIndex);
	SnmpOid nameOid = name.toOid();
	long length = paramsIndex.length();

	//RowStatus
	SnmpOid oidRowStatus = new SnmpOid("1.3.6.1.6.3.12.1.3.1.7");
	oidRowStatus.append(length);
	oidRowStatus.append(nameOid);
	EnumRowStatus destroy = new EnumRowStatus("destroy");
	if(isDebugOn())
	    debug("removeParams", oidRowStatus.toString());
	list.addVarBind(new SnmpVarBind(oidRowStatus, 
					destroy.toSnmpValue()));

	// Send the SET request
	SnmpRequest request = session.snmpSetRequest(null, list);

	boolean completed = request.waitForCompletion(session.getDefaultPeer().getTimeout());
	if (completed == false) {
	  if (isTraceOn())
	      trace("register", "Request timed out: check reachability of agent.\n");
	  throw new SnmpStatusException(SnmpDefinitions.snmpReqTimeout, 
					0);
	}

	// Check the status
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    if (isTraceOn()) {
		trace("register","************* Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
		trace("register","Error index = " + request.getErrorIndex());
		throw new SnmpStatusException(errorStatus,
					      request.getErrorIndex());
	    }
	} else {
	    if (isDebugOn()) {
		SnmpVarBindList result = request.getResponseVarBindList();
		debug("register","Result: \n" + result.varBindListToString());
	    }
	}
    }



    public static void setTarget(SnmpSession session, 
			  SnmpTargetData data) 
	throws SnmpStatusException {

	SnmpVarBindList list = new SnmpVarBindList();

	//Name Index
	SnmpString name = new SnmpString(data.targetName);
	SnmpOid nameOid = name.toOid();
	long length = data.targetName.length();

	// Domain OID
	SnmpOid oidTDomain = new SnmpOid("1.3.6.1.6.3.12.1.2.1.2");
	SnmpOid domain = new SnmpOid(data.domain);
	oidTDomain.append(length);
	oidTDomain.append(nameOid);
	if(isDebugOn())
	    debug("setTargets", oidTDomain.toString());
	list.addVarBind(new SnmpVarBind(oidTDomain, domain));

	// Address
	SnmpOid oidAddress = new SnmpOid("1.3.6.1.6.3.12.1.2.1.3");
	
	SnmpIpDomainsAddress translator = null;
	try {
	    translator = new SnmpIpDomainsAddress(data.domain, data.address);
	}catch(Exception e) {
	    e.printStackTrace();
	}

	SnmpString address = new SnmpString(translator.getMibAddress());
	oidAddress.append(length);
	oidAddress.append(nameOid);
	if(isDebugOn())
	    debug("setTargets", oidAddress.toString());
	list.addVarBind(new SnmpVarBind(oidAddress, address));

	// Timeout
	SnmpOid oidTimeout = new SnmpOid("1.3.6.1.6.3.12.1.2.1.4");
	SnmpInt timeout = new SnmpInt(data.timeout);
	oidTimeout.append(length);
	oidTimeout.append(nameOid);
	if(isDebugOn())
	    debug("setTargets", oidTimeout.toString());
	list.addVarBind(new SnmpVarBind(oidTimeout, timeout));

	// Retry count
	SnmpOid oidRetry = new SnmpOid("1.3.6.1.6.3.12.1.2.1.5");
	SnmpInt retry = new SnmpInt(data.retry);
	oidRetry.append(length);
	oidRetry.append(nameOid);
	if(isDebugOn())
	    debug("setTargets", oidRetry.toString());
	list.addVarBind(new SnmpVarBind(oidRetry, retry));

	// Tag List
	SnmpOid oidTags = new SnmpOid("1.3.6.1.6.3.12.1.2.1.6");
	SnmpString tags = new SnmpString(data.tags);
	oidTags.append(length);
	oidTags.append(nameOid);
	if(isDebugOn())
	    debug("setTargets", oidTags.toString());
	list.addVarBind(new SnmpVarBind(oidTags, tags));

	// Params Reference Index
	SnmpOid oidParams = new SnmpOid("1.3.6.1.6.3.12.1.2.1.7");
	SnmpString params = new SnmpString(data.params);
	oidParams.append(length);
	oidParams.append(nameOid);
	if(isDebugOn())
	    debug("setTargets", oidParams.toString());
	list.addVarBind(new SnmpVarBind(oidParams, params));

	// Storage type
	SnmpOid oidStorage = new SnmpOid("1.3.6.1.6.3.12.1.2.1.8");
	SnmpInt storage = new SnmpInt(data.storageType);
	oidStorage.append(length);
	oidStorage.append(nameOid);
	if(isDebugOn())
	    debug("setParams", oidStorage.toString());
	list.addVarBind(new SnmpVarBind(oidStorage, storage));

	// RowStatus
	SnmpOid oidRowStatus = new SnmpOid("1.3.6.1.6.3.12.1.2.1.9");
	oidRowStatus.append(length);
	oidRowStatus.append(nameOid);
	EnumRowStatus createAndGo = new EnumRowStatus("createAndGo");
	if(isDebugOn())
	    debug("setParams", oidRowStatus.toString());
	list.addVarBind(new SnmpVarBind(oidRowStatus, 
					createAndGo.toSnmpValue()));


	// Send the SET request
	SnmpRequest request = session.snmpSetRequest(null, list);

	// Wait for completion
	boolean completed = request.waitForCompletion(session.getDefaultPeer().getTimeout());
	if (completed == false) {
	  if (isTraceOn())
	      trace("register", "Request timed out: check reachability of agent.");
	  throw new SnmpStatusException(SnmpDefinitions.snmpReqTimeout, 
				0);
	}

	// Check the status
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    if (isTraceOn()) {
		trace("register","Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
		trace("register","Error index = " + request.getErrorIndex());
		throw new SnmpStatusException(errorStatus,
					      request.getErrorIndex());
	    }
	} else {
	    if (isDebugOn()) {
		SnmpVarBindList result = request.getResponseVarBindList();
		debug("register","Result: \n" + result.varBindListToString());
	    }
	}
    }

    public static void removeTarget(SnmpSession session,
				    String targetIndex) 
	throws SnmpStatusException {
	
	SnmpVarBindList list = new SnmpVarBindList();

	//Name Index
	SnmpString name = new SnmpString(targetIndex);
	SnmpOid nameOid = name.toOid();
	long length = targetIndex.length();

	//RowStatus
	SnmpOid oidRowStatus = new SnmpOid("1.3.6.1.6.3.12.1.2.1.9");
	oidRowStatus.append(length);
	oidRowStatus.append(nameOid);
	EnumRowStatus destroy = new EnumRowStatus("destroy");
	if(isDebugOn())
	    debug("removeTargets", oidRowStatus.toString());
	list.addVarBind(new SnmpVarBind(oidRowStatus, 
					destroy.toSnmpValue()));

	// Send the SET request
	SnmpRequest request = session.snmpSetRequest(null, list);

	boolean completed = request.waitForCompletion(session.getDefaultPeer().getTimeout());
	if (completed == false) {
	  if (isTraceOn())
	      trace("register", "Request timed out: check reachability of agent.\n");
	  throw new SnmpStatusException(SnmpDefinitions.snmpReqTimeout, 
					0);
	}

	// Check the status
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    if (isTraceOn()) {
		trace("register","************* Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
		trace("register","Error index = " + request.getErrorIndex());
		throw new SnmpStatusException(errorStatus,
					      request.getErrorIndex());
	    }
	} else {
	    if (isDebugOn()) {
		SnmpVarBindList result = request.getResponseVarBindList();
		debug("register","Result: \n" + result.varBindListToString());
	    }
	}
    }
	
    // TRACES & DEBUG
    //---------------
    
    static boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    static void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    static void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    static boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    static void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    static void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    static void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    static void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    static String dbgTag = "SnmpTargetsParamsHandler"; 
}
