/*
 * @(#)file      SnmpTargetData.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.target;

import com.sun.management.snmp.rfc2573.internal.target.SnmpPeerFactory;
import com.sun.management.snmp.rfc2573.target.SnmpAddress;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.SnmpEngine;
import com.sun.jdmk.trace.Trace;

public class SnmpTargetData implements Cloneable {
    public String targetName = null;
    public String domain = null;
    public SnmpAddress address = null;
    public int timeout = -1;
    public int retry = -1;
    public String tags = null;
    public SnmpParamsData params = null;
    public int storageType = -1;
    public SnmpPeer authoritativePeer = null;
    public SnmpPeer nonAuthoritativePeer = null;
    private SnmpEngine engine = null;
    private SnmpPeerFactory factory = null;
    private String dbgTag = "SnmpTargetData";
    public SnmpTargetData(SnmpEngine engine,
			  SnmpPeerFactory factory) {
	this .engine = engine;
	this.factory = factory;
    }

    public synchronized void deactivateAuthoritativePeer() {
	authoritativePeer = null;
    }
    
    public synchronized void deactivateNonAuthoritativePeer() {
	nonAuthoritativePeer = null;
    }

    public synchronized void activateAuthoritativePeer() throws IllegalArgumentException {
	if(authoritativePeer == null) {
	    try {
		authoritativePeer = factory.createAuthoritativePeer(engine,
								    params,
								    address);
	    }catch(Exception e) {
		e.printStackTrace();
		throw new IllegalArgumentException(e.getMessage());
	    }
	    if(authoritativePeer == null) {
		if(isDebugOn())
		    debug("activateAuthoritativePeer", "WARNING. PEER NOT ACTIVATED. No peer found for message processing model and or security model.");
		throw new IllegalArgumentException("WARNING. PEER NOT ACTIVATED. No peer found for message processing model and or security model.");
	    }
	}
    }
    
    public synchronized void activateNonAuthoritativePeer() throws IllegalArgumentException {
	if(nonAuthoritativePeer == null) {
	    try {
		nonAuthoritativePeer = 
		    factory.createNonAuthoritativePeer(engine, 
						       params,
						       address);
	    }catch(Exception e) {
		e.printStackTrace();
		throw new IllegalArgumentException(e.getMessage());
	    }
	    if(nonAuthoritativePeer == null) {
		if(isDebugOn())
		    debug("activateNonAuthoritativePeer", "WARNING. PEER NOT ACTIVATED. No peer found for message processing model and or security model.");
		throw new IllegalArgumentException("WARNING. PEER NOT ACTIVATED. No peer found for message processing model and or security model.");
	    }    
	}
    }

    public Object clone() throws CloneNotSupportedException {
	return super.clone();
    }
    
    public String toString() {
	return targetName + "," + 
	    domain + "," + 
	    address + "," + 
	    timeout + "," + 
	    retry + "," + 
	    tags + "," +
	    params.paramsName + "," +
	    storageType;
    }

    private boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP);
    }

    private void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    private boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP);
    }

    private void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
}
