/*
 * @(#)file      SnmpTargetAddrEntryImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.2
 * @(#)date      01/12/20
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.target.mib;



// java imports
//
import java.io.Serializable;
import java.net.UnknownHostException;
import java.net.InetAddress;

// jmx imports
//
import com.sun.management.snmp.SnmpStatusException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
// jdmk imports
//
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.ServiceName;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.EnumRowStatus;

//Contrib imports
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.target.SnmpTargetData;

public class SnmpTargetAddrEntryImpl extends SnmpTargetAddrEntry {
    //Constructor called from the SNMP MIB.
    SnmpTargetLcd lcd = null;
    SnmpTargetData data = null;
    public SnmpTargetAddrEntryImpl(SnmpMib theMib, 
				   SnmpTargetLcd lcd) {
	super(theMib);
	this.lcd = lcd;
    }
    
    //Constructor called when instanciating from conf.
    public SnmpTargetAddrEntryImpl(SnmpMib theMib, 
				   SnmpTargetLcd lcd, 
				   SnmpTargetData data) throws IllegalArgumentException, UnknownHostException {
	super(theMib);
	this.lcd = lcd;
	this.data = data;
	
	SnmpTargetAddrName = data.targetName;
	SnmpTargetAddrTDomain = data.address.getDomainOid();
	byte[] bytes = data.address.getAddress();

	Byte[] bigByte = new Byte[bytes.length];
	for(int i = 0; i < bytes.length; i++)
	    bigByte[i] = new Byte(bytes[i]);
	
	SnmpTargetAddrTAddress = bigByte;
	SnmpTargetAddrTimeout = new Integer(data.timeout);
	SnmpTargetAddrRetryCount = new Integer(data.retry);
	byte[] b = data.tags.getBytes();
	int size = b.length;
	SnmpTargetAddrTagList = new Byte[size];
	for(int i = 0; i < size; i++)
	    SnmpTargetAddrTagList[i] = new Byte(b[i]);
	SnmpTargetAddrParams = data.params.paramsName;
	SnmpTargetAddrStorageType  = new EnumSnmpTargetAddrStorageType(data.storageType);
    }
    
    /**
     * Checker for the "SnmpTargetAddrRowStatus" variable.
     */
    public void setSnmpTargetAddrRowStatus(EnumSnmpTargetAddrRowStatus x) throws SnmpStatusException {
	//Do it here or NOT.
	if(isDebugOn())
	    debug("setSnmpTargetAddrRowStatus", "Received row status is : " + x.intValue());
	if(x.intValue() == EnumRowStatus.active) {
	    try {
		
		//Make the tags.
		int size = SnmpTargetAddrTagList.length;
		byte[] tags = new byte[size];
		
		for(int i = 0; i < size; i++) 
		    tags[i] = SnmpTargetAddrTagList[i].byteValue();
		lcd.addTarget(SnmpTargetAddrName,
			      SnmpTargetAddrTDomain,
			      SnmpTargetAddrTAddress,
			      SnmpTargetAddrTimeout.intValue(),
			      SnmpTargetAddrRetryCount.intValue(),
			      new String(tags),
			      SnmpTargetAddrParams,
			      SnmpTargetAddrStorageType.intValue());
		if(isDebugOn())
		    debug("setSnmpTargetAddrRowStatus",
			  "Row status active. addTarget done.");
		super.setSnmpTargetAddrRowStatus(x);
	    }catch(IllegalArgumentException e) {
		
		throw new SnmpStatusException(e.getMessage());
	    }
	}
    }

    public void addInMBeanServer(MBeanServer server) {
	try {
	    //Use the prefix.
            server.registerMBean(this, createObjectName());
        } catch (Exception e) {
	    if(isTraceOn())
		trace("addInMBeanServer", e.toString());
        }
    }
    
    protected ObjectName createObjectName() 
	throws MalformedObjectNameException {
	return new ObjectName("TableSnmpTargetAddrTable:name=com.sun.management.snmp.target.mib.SnmpTargetAddrEntry" + ",SnmpTargetAddrName=" + SnmpTargetAddrName);
    }

     // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpTargetAddrEntryImpl"; 
}
