/*
 * @(#)file      SnmpTargetParamsEntryImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.2
 * @(#)date      01/12/20
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.target.mib;



// java imports
//
import java.io.Serializable;

// jmx imports
//
import com.sun.management.snmp.SnmpStatusException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
// jdmk imports
//
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.ServiceName;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.EnumRowStatus;

//Contrib imports
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;

public class SnmpTargetParamsEntryImpl extends SnmpTargetParamsEntry {
    //Constructor called from the SNMP MIB.
    SnmpTargetLcd lcd = null;
    SnmpParamsData data = null;
     public SnmpTargetParamsEntryImpl(SnmpMib theMib, 
				      SnmpTargetLcd lcd) {
	 super(theMib);
	 this.lcd = lcd;
     }

    //Constructor called when instanciating from conf.
    public SnmpTargetParamsEntryImpl(SnmpMib theMib, 
				     SnmpTargetLcd lcd, 
				     SnmpParamsData data) {
	super(theMib);
	this.lcd = lcd;
	this.data = data;
	
	SnmpTargetParamsName = data.paramsName;
	SnmpTargetParamsSecurityName = data.securityName;
	SnmpTargetParamsSecurityLevel = new EnumSnmpTargetParamsSecurityLevel(data.securityLevel + 1);
	SnmpTargetParamsStorageType = new EnumSnmpTargetParamsStorageType(data.storageType);
	SnmpTargetParamsSecurityModel = new Integer(data.securityModel);
	SnmpTargetParamsMPModel = new Integer(data.mpm);
    }
    /**
     * Checker for the "SnmpTargetParamsRowStatus" variable.
     */
    public void setSnmpTargetParamsRowStatus(EnumSnmpTargetParamsRowStatus x) throws SnmpStatusException {
	//Do it here or NOT.
	if(isDebugOn())
	    debug("setSnmpTargetParamsRowStatus", "Received row status is : " + x.intValue());
	if(x.intValue() == EnumRowStatus.active) {
	    try {
		lcd.addParams(SnmpTargetParamsName,
			      SnmpTargetParamsMPModel.intValue(),
			      SnmpTargetParamsSecurityModel.intValue(),
			      SnmpTargetParamsSecurityName,
			      SnmpTargetParamsSecurityLevel.intValue(),
			      SnmpTargetParamsStorageType.intValue());
		if(isDebugOn())
		    debug("setSnmpTargetParamsRowStatus",
			  "Row status active. addParams done.");
		super.setSnmpTargetParamsRowStatus(x);
	    }catch(IllegalArgumentException e) {
		
		throw new SnmpStatusException(e.getMessage());
	    }
	}
    }

    public void addInMBeanServer(MBeanServer server) {
	try {
	    //Use the prefix.
            server.registerMBean(this, createObjectName());
        } catch (Exception e) {
	    if(isTraceOn())
		trace("addInMBeanServer", e.toString());
        }
    }
    
    protected ObjectName createObjectName() 
	throws MalformedObjectNameException {
	return new ObjectName("TableSnmpTargetParamsTable:name=com.sun.management.snmp.target.mib.SnmpTargetParamsEntry" + ",SnmpTargetParamsName=" + SnmpTargetParamsName);
    }

     // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpTargetParamsEntryImpl"; 
}
