/*
 * @(#)file      InterfacesImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */

// java import
//

// RI import
//
import javax.management.MBeanServer;

// jdmk import
//
import com.sun.management.snmp.agent.SnmpMib;


/**
 * The InterfacesImpl class provides an simple implementation of the "Interfaces" group.
 * It creates two entries in the "IfTable" at instantiation time and add them into the MBean server. 
 */

public class InterfacesImpl extends Interfaces {

    // MBean properties.
    //
    private static IfEntryImpl le0 = null;
    private static IfEntryImpl lo0 = null;

    /**
     * Constructors.
     */
    public InterfacesImpl(SnmpMib myMib) {
        super(myMib);
        init(myMib);
    }

    public InterfacesImpl(SnmpMib myMib, MBeanServer server) {
        super(myMib, server);
        init(myMib, server);
    }

    private void init(SnmpMib myMib) {
        
        IfNumber = new Integer(2);

        le0 = new IfEntryImpl(myMib);
        le0.IfIndex = new Integer(1);
        le0.IfMtu = new Integer(1500);
        le0.IfType = new EnumIfType(6); // "ethernet-csmacd"
        le0.IfDescr = "le0";
        le0.IfOperStatus = new EnumIfOperStatus(1); // "up"

        lo0 = new IfEntryImpl(myMib);
        lo0.IfIndex = new Integer(2);
        lo0.IfMtu = new Integer(8232);
        lo0.IfType = new EnumIfType(24); // "softwareLoopback"
        lo0.IfDescr = "lo0";
        lo0.IfOperStatus = new EnumIfOperStatus(1); // "up"

        // Register le0 and lo0 into the ifTable.
        //
        try {
            IfTable.addEntry(le0);
            IfTable.addEntry(lo0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void init(SnmpMib myMib, MBeanServer server) {

        init(myMib);
        
        // Register le0 and lo0 into the MBean server.
        //
        le0.addInMBeanServer(server);
        lo0.addInMBeanServer(server);
    }
    
    public static IfEntryImpl find(int ifIndex) {
        if (ifIndex == le0.IfIndex.intValue()) {
            return le0;
        } else if (ifIndex == lo0.IfIndex.intValue()) {
            return lo0;
        } else {
            return null;
        }
    }
}



