/*
 * @(#)file      SnmpV3Proxy.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.11
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.SocketException;
// RI import
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineParameters;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpUnknownModelException;

// jdmk import
//
import com.sun.jdmk.TraceManager;
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpV3AdaptorServer;

import com.sun.management.snmp.agent.SnmpTrapForwarder;

import com.sun.management.internal.snmp.SnmpSecuritySubSystem;
import com.sun.management.snmp.usm.*;

import com.sun.management.comm.SnmpV3InformRequest;
import com.sun.management.snmp.rfc2573.SnmpJdmkRfc2573ProxyForwarder;
import com.sun.management.snmp.rfc2573.SnmpJdmkRfc2573NotificationOriginator;


/**
 * This class is a JDMK 5.0 standalone agent that allows manager to register for trap and sub agents to register for forwarding. It follows the RFC 2573 SNMP V3 applications.
 */
public class SnmpV3Proxy {
    static String dbgTag = "SnmpV3Proxy";
    public static void main(String[] args) {
	
	SnmpV3AdaptorServer adaptor = null;
	MBeanServer server = null;
        ObjectName htmlObjName = null;
        ObjectName snmpObjName = null;
        ObjectName mibObjName= null;
        int htmlPort = 8083;
	try {
            TraceManager.parseTraceProperties();
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", "Default trace is on");
        } catch(java.io.IOException e) {
            e.printStackTrace();
        }
	
        try {
            server = MBeanServerFactory.createMBeanServer();
            String domain = server.getDefaultDomain();
	    
            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain + ":class=HtmlAdaptorServer,protocol=html,port=" + htmlPort);
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "Agent", "main", 
                       "Adding HTML adaptor to MBean server with name \n\t" + htmlObjName);
            java.lang.System.out.println("NOTE: HTML adaptor is bound on TCP port " + htmlPort);
            HtmlAdaptorServer htmlAdaptor = new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();
			      
	    SnmpEngineParameters params = new SnmpEngineParameters();
	    params.setEngineId(SnmpEngineId.createEngineId("localhost:8085"));
	    adaptor = new SnmpV3AdaptorServer(params,
					      null,
					      null,
					      8085,
					      null);
	    // Enable the SET version V1 and V2
	    adaptor.enableSnmpV1V2SetRequest();
	    
	    adaptor.start();
	    
	    java.lang.System.out.println("NOTE: SNMP adaptor is bound on TCP port 8085");
	    String addrFile = null;
	    addrFile = (String) java.lang.System.getProperty("com.sun.management.snmp.rfc2573.target.addr.file");
	    if(addrFile == null) {
		if(isTraceOn())
		    trace("main", "No address configuration file, no persistent configuration.");
	    }
	    
	    String paramsFile = null;
	    paramsFile = (String) java.lang.System.getProperty("com.sun.management.snmp.rfc2573.target.params.file");
	    if(paramsFile == null) {
		if(isTraceOn())
		    trace("main", "No params configuration file, no persistent configuration.");
	    }
	    
	    String proxyFile = null;
	    proxyFile = (String) java.lang.System.getProperty("com.sun.management.snmp.rfc2573.proxy.file");
	    if(proxyFile == null) {
		if(isTraceOn())
		    trace("main", "No proxy configuration file, no persistent configuration.");
	    }
	    
	    SnmpJdmkRfc2573ProxyForwarder proxy = 
		new SnmpJdmkRfc2573ProxyForwarder(adaptor,
						  server,
						  paramsFile,
						  addrFile,
						  proxyFile);
	    proxy.start();
	    java.lang.System.out.println("    SNMP V3 request forwarding initilialized");

	    // Persistent file to use for Notifications
	    String notifFile = null;
	    notifFile = (String) java.lang.System.getProperty("com.sun.management.snmp.rfc2573.notification.file");
	    if(notifFile == null) {
		if(isTraceOn())
		    trace("main", "No notification configuration file, no persistent configuration.");
	    }

	    // Notification MIB originator
	    SnmpJdmkRfc2573NotificationOriginator notif =
		new SnmpJdmkRfc2573NotificationOriginator(adaptor,
							  server,
							  notifFile,
							  proxy.getTargetLcd());
	    notif.start();
	    
	    // Notification forwarding.
	    final SnmpTrapForwarder trapForwarder = 
		new SnmpNotificationForwarder(adaptor,
					      8086, notif);
	    // Trap forwarder can forward as notification originator or 
	    // as proxy.
	    trapForwarder.forwardAsProxy();
	    
	    // The SnmpTrapForwarder is an MBean, register it in the 
	    // MBeanServer
	    //
	    final ObjectName n = 
		new ObjectName("master_agent:class=SnmpTrapForwarder," + 
			       "port=8086");
	    server.registerMBean(trapForwarder, n);

	    // Start the forward. No specific configuration has been 
	    // provided, it will forward to SNMP V1 and SNMP V2 managers 
	    // only (thanks to IPAcl policy).
	    //
	    try{
		trapForwarder.start();
	    } catch(SocketException e) { e.printStackTrace(); }
	    

	    java.lang.System.out.println("    SNMP V3 notification forwarding initilialized");
	    java.lang.System.out.println("    SNMP V3 proxy initilialized");
	    
	}catch(Exception e) {
	    java.lang.System.out.println(e);
	}
    }

    // TRACES & DEBUG
    //---------------
    
    static boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    static void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    static void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    static boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    static void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    static void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }
    
    static void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    static void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }

}
