/*
 * @(#)file      JmxMBeanAttrEntryImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/07
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import java.io.Serializable;
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.agent.SnmpMib;

/**
 * The class is used for implementing the "JmxMBeanAttrEntry" conceptual row.
 * An instance of this class mirrors the value of an attribute of an 
 * MBean mirrored in the jmxMBeanTable.
 *
 * In this implementation, the JmxMBeanAttrTable is a virtual table - which
 * is computed on demand (see JmxMBeanAttrTableMeta).
 * JmxMBeanAttrEntryImpl objects are thus transient object which are created
 * on the fly, and then later reclaimed by the garbage collector when they
 * are no longer needed. To avoid constant recomputation, the 
 * JmxMBeanAttrEntryImpl are cached into their corresponding JmxMBeanEntryImpl.
 */
public class JmxMBeanAttrEntryImpl extends JmxMBeanAttrEntry {

    /**
     * The JmxMBeanEntryImpl to which this attribute belongs.
     **/
    final private JmxMBeanEntryImpl mbeanEntry;

    /**
     * Constructor for the "JmxMBeanAttrEntry" conceptual row.
     */
    public JmxMBeanAttrEntryImpl(SnmpMib myMib, JmxMBeanEntryImpl mbeanEntry) {
	super(myMib);
	this.mbeanEntry = mbeanEntry;
    } 

    /**
     * Getter for the "JmxMBeanAttrStringValue" variable.
     * This is the actual value of the attribute, which is never cached.
     * We need to truncate the name to the maximum length allowed by the
     * JmxMBeanAttrNameTC.
     */
    public String getJmxMBeanAttrStringValue() throws SnmpStatusException {
 	try {
	    final MBeanServer s = mbeanEntry.getMibServer();
	    final Object val = 
		s.getAttribute(mbeanEntry.getObjectName(),JmxMBeanAttrName);
	    return JMX_MBEAN_SERVER_MIB_Impl.
		validJmxMBeanAttrStringValueTC(String.valueOf(val));
	} catch (Exception x) {
	    // If we get an exception, we return the toString() of the
	    // exception.
	    //
	    return JMX_MBEAN_SERVER_MIB_Impl.
		validJmxMBeanAttrStringValueTC(String.valueOf(x));
	}
    }

    /**
     * Getter for the "JmxMBeanAttrName" variable.
     * We need to truncate the name to the maximum length allowed by the
     * JmxMBeanAttrNameTC.
     */
    public String getJmxMBeanAttrName() throws SnmpStatusException {
	return JMX_MBEAN_SERVER_MIB_Impl.
	    validJmxMBeanAttrNameTC(JmxMBeanAttrName);
    }
}
