/*
 * @(#)file      StandAloneAgent.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.30
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import com.sun.management.comm.SnmpAdaptorServer;

/**
 * The StandAloneAgent class provides a standalone SNMP agent.
 * A subset of MIB II (RFC1213) is loaded and initialized.
 *
 * The default port number of the SNMP adaptor is 8086.
 * As such, you do not need to be root to start the agent.
 * However, this default value can be changed by specifying
 * a port number in the command-line.
 * For instance, if you want to use port 44450, start the agent as follows:
 *      java StandAloneAgent 44450
 */

public class StandAloneAgent {

    /**
     * The main method allows you to specify the following optional
     * command-line parameters:
     *  - port: port number to use for binding the SNMP adaptor.
     *
     * If you don't specify any port number, the default value is 8086.
     */
    public static void main(String argv[]) {

        // If not specified, the default port is 8086.
        //
        int port = 8086;

        // The port can be specified in the command line.
        //
        if (argv.length != 0) {
            if (argv.length == 1) {
                try {
                    port = Integer.parseInt(argv[0]);
                } catch (NumberFormatException e) {
                    java.lang.System.err.println("Exception occurred: " + e);
                    e.printStackTrace();
                    java.lang.System.exit(1);
                }
            } else {
                usage();
                java.lang.System.exit(1);
            }
        }

        try {

            // Create and start the SNMP adaptor.
            //
            SnmpAdaptorServer snmpAdaptor = new SnmpAdaptorServer(port);
            java.lang.System.out.println("\nAdding SNMP adaptor using port " +
                                         port);
            snmpAdaptor.start();

            // Create and initialize the subset of MIB II.
            // The name of the generated class depends on the name
            // of the module in the mib. We create here an instance
            // of the customized MIB.
            //
            RFC1213_MIB mib2 = new RFC1213_MIB_IMPL();
            mib2.init();
            java.lang.System.out.println("\nInitializing the MIB " +
                                         mib2.getMibName());

            // Bind the SNMP adaptor to the MIB in order to make the MIB
            // accessible through the SNMP protocol.
            //	    
            mib2.setSnmpAdaptor(snmpAdaptor);

        } catch (Exception e) {
            java.lang.System.err.println("Exception occurred: " + e);
            e.printStackTrace();
        }
    }

    /**
     * Return usage of the program.
     */
    public static void  usage() {
        java.lang.System.out.println("\nUsage: java StandAloneAgent <port>");
        java.lang.System.out.println("where");
        java.lang.System.out.println("\t-port: port number to use.");
        java.lang.System.out.println("\t       This parameter is not " +
                                     "mandatory.");
        java.lang.System.out.println("\t       If not specified, the " +
                                     "default port is 8086.\n");
    }
}
