/*
 * @(#)file      DemoImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.3
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;

// jmx imports
//
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpStatusException;

// jdmk imports
//
import com.sun.management.snmp.agent.SnmpMib;

/**
 * This class is a customized implementation of the "Demo" group.
 * It shows how to override the constructors of the group in order
 * to enable the remote creation of entries by a SNMP manager.
 *
 * Since the demoTable contains a columnar-object which has a RowStatus
 * syntax, the dynamic creation of rows will be entirely taken in charge
 * by mibgen.
 *
 */
public class DemoImpl extends Demo {

    /**
     * We override the constructor in order to enable the remote creation of
     * rows in the DemoTable.
     */
    public DemoImpl(SnmpMib myMib) {
	super(myMib);
	// Instantiate our own implementation of the DemoTable
	//
	DemoTable = new TableDemoTableImpl(myMib, this);

	// Enable remote entry creation in the DemoTable
	//
	DemoTable.setCreationEnabled(true);
    }


    /**
     * We override the constructor in order to instantiate our own 
     * implementation of the DemoTable. Then we enable the remote creation
     * of rows in the DemoTable
     */
    public DemoImpl(SnmpMib myMib, MBeanServer server) {
	super(myMib,server);

	// Instantiate our own implementation of the DemoTable
	//
	DemoTable = new TableDemoTableImpl(myMib, server, this);

	// Enable remote entry creation in the DemoTable
	//
	DemoTable.setCreationEnabled(true);
    }

    /**
     * This method will be called by our customized implementation of
     * the DemoTable when a row is added.
     **/
    public void addRowCb() {
	DemoInteger = new Integer(DemoInteger.intValue() + 1);
    }

    /**
     * This method will be called by our customized implementation of
     * the DemoTable when a row is removed.
     **/
    public void removeRowCb() {
	DemoInteger =  new Integer(DemoInteger.intValue() - 1);
    }
}
